/*
 * Copyright (C) 2012, Eric Wong <normalperson@yhbt.net>
 * License: GPLv3 or later (see COPYING for details)
 */
#include "cmogstored.h"
#include "digmd5.h"

struct md5_ctx * mog_md5_new(void)
{
	struct md5_ctx *ctx = mog_cachealign(sizeof(struct md5_ctx));

	md5_init_ctx(ctx);

	return ctx;
}

enum mog_md5_next mog_md5_read(struct md5_ctx *ctx, int fd)
{
	size_t len;
	void *buf = mog_fsbuf_get(&len);
	size_t i = 1024;

	while (--i > 0) {
		ssize_t r = read(fd, buf, len);

		if (r > 0) { /* most likely */
			md5_process_bytes(buf, r, ctx);
		} else if (r == 0) {
			/* wait for user to call mog_md5_hex() */
			return MOG_MD5_EOF;
		} else {
			assert(r < 0 && errno && "buggy read(2)?");
			/* may happen on crazy FSes */
			if (errno == EINTR)
				continue;
			/* bail on EAGAIN, not possible on regular files */
			return MOG_MD5_ERROR;
		}
	}

	return MOG_MD5_CONTINUE;
}

void mog_md5_hex(struct md5_ctx *ctx, char *buf, size_t len)
{
	static const char hex[] = "0123456789abcdef";
	unsigned char result[16];
	unsigned char *b = result;
	char *out = buf;
	int i;

	md5_finish_ctx(ctx, result);

	assert(len >= (sizeof(result) * 2) && "hex buffer too small");

	for (i = 0; i < sizeof(result); i++) {
		*out++ = hex[*b >> 4];
		*out++ = hex[*b & 0x0f];
		b++;
	}
}
